// The MIT License

// Copyright (c) 2009 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Author: Kermin Fleming kfleming@mit.edu
// De-AWBized: Richard Uhler ruhler@mit.edu

import FIFO::*;
import FIFOF::*;
import GetPut::*;

import AudioProcessorTypes::*;

interface Depadder#(numeric type blockSize);
  interface AudioProcessor pipeline;
  interface Put#(Bit#(TLog#(blockSize))) padding;
endinterface

module mkDepadder(Depadder#(blockSize))
  provisos(Add#(1, TLog#(blockSize), TAdd#(1, TLog#(blockSize))));

  FIFO#(AudioProcessorUnit) inFifo <- mkFIFO();
  FIFO#(AudioProcessorUnit) outFifo <- mkFIFO();  
  FIFO#(Bit#(TLog#(blockSize))) paddingFifo <- mkFIFO();
  Reg#(Bit#(TAdd#(1, TLog#(blockSize)))) blockCount <- mkReg(fromInteger(valueof(blockSize)));
  

  rule processSample(inFifo.first() matches tagged Sample .sample);
    if(blockCount - 1 == 0)
      begin
        blockCount <= fromInteger(valueof(blockSize));
        paddingFifo.deq();
      end    
    else
      begin
        blockCount <= blockCount - 1; 
      end

    if(blockCount > zeroExtend(paddingFifo.first()))
      begin
        outFifo.enq(inFifo.first());
      end

    inFifo.deq();
  endrule
 
  rule endOfFile(inFifo.first() matches tagged EndOfFile);
    outFifo.enq(inFifo.first());
    inFifo.deq();
  endrule  

  interface AudioProcessor pipeline;
    method Action putSampleInput(AudioProcessorUnit in) = inFifo.enq(in);
    method ActionValue#(AudioProcessorUnit) getSampleOutput();
        outFifo.deq();
        return outFifo.first();
    endmethod
  endinterface
  
  interface padding = fifoToPut(paddingFifo);

endmodule
